#define SERIAL_DATA Serial1
#define SERIAL_DEBUG Serial

const byte LED_R = 16;
const byte LED_Y = 15;
const byte LED_G = 14;

byte currentState = 0;
bool autoSwitch = true;

void setup() {
  pinMode(LED_R, OUTPUT);
  pinMode(LED_Y, OUTPUT);
  pinMode(LED_G, OUTPUT);
  setState(4);
  
  SERIAL_DATA.begin(9600);
  SERIAL_DEBUG.begin(9600);
}

void loop() {
  readSerial();
  if(autoSwitch) setState(calcCurrentState());
  setState(currentState);
}

void setState(byte state) {
  if(state <= 3 && state == currentState) return;
  switch(state) {
    case 0: // czerwone
      setLeds(true, false, false);
      break;
    case 1: // czerwone -> zielone (przygotowanie)
      setLeds(true, true, false);
      break;
    case 2: // zielone
      setLeds(false, false, true);
      break;
    case 3: // zielone -> czerwone (przygotowanie)
      setLeds(false, true, false);
      break;
    default: // jazda wg. znaków (migające pomarańczowe)  
      setLeds(false, (millis() % 1600 < 800), false);
      currentState = 4;
      return;
  }
  currentState = state;
}

void setLeds(bool red, bool yellow, bool green) {
  digitalWrite(LED_R, red);
  digitalWrite(LED_Y, yellow);
  digitalWrite(LED_G, green);
}

byte calcCurrentState() {
  unsigned long time = millis() % 10000;
  if(time < 4000) return 0;
  if(time < 5000) return 1;
  if(time < 9000) return 2;
  return 3;
}

void readSerial() {
  char temp;
  String data = "";
  unsigned long timeout = millis();
  while(millis() - timeout <= 100) {
    while(SERIAL_DATA.available()) {
      temp = SERIAL_DATA.read();
      timeout = millis();
      if(temp == '\r' || temp == '\n') continue;
      data += temp;
    }
  }

  if(data.length() == 0) return;

  data.toUpperCase();
  
  SERIAL_DEBUG.print("GOT: \"");
  SERIAL_DEBUG.print(data);
  SERIAL_DEBUG.println("\"");

  if(data.equals("AT")) {
    SERIAL_DATA.println("OK");
  } else if(data.startsWith("AT+")) {
    data = data.substring(3);
    int paramsOutput[3];
    
    if(data.equals("AUTOTOGGLE")) {
      autoSwitch = !autoSwitch;
      SERIAL_DATA.print("AT+AUTO=");
      SERIAL_DATA.println(autoSwitch);
    } else if(data.startsWith("STATE")) {
      byte expectedParamsCount = 1;
      byte result = getParams(data.substring(5), expectedParamsCount, paramsOutput);
      switch(result) {
        case 0:
          autoSwitch = false;
          setState(paramsOutput[0]);
        case 1:
          SERIAL_DATA.print("AT+STATE=");
          SERIAL_DATA.println(currentState);
          break;
        default:
          SERIAL_DATA.print("ERROR: ");
          SERIAL_DATA.println(result);
      }
    } else if(data.startsWith("AUTO")) {
      byte expectedParamsCount = 1;
      byte result = getParams(data.substring(4), expectedParamsCount, paramsOutput);
      switch(result) {
        case 0:
          autoSwitch = paramsOutput[0];
        case 1:
          SERIAL_DATA.print("AT+AUTO=");
          SERIAL_DATA.println(autoSwitch);
          break;
        default:
          SERIAL_DATA.print("ERROR: ");
          SERIAL_DATA.println(result);
      }
    } else if(data.startsWith("LEDS")) {
      byte expectedParamsCount = 3;
      byte result = getParams(data.substring(4), expectedParamsCount, paramsOutput);
      switch(result) {
        case 0:
          setLeds(paramsOutput[0],paramsOutput[1],paramsOutput[2]);
        case 1:
          autoSwitch = false;
          SERIAL_DATA.print("AT+LEDS=");
          SERIAL_DATA.print(digitalRead(LED_R));
          SERIAL_DATA.print(" ");
          SERIAL_DATA.print(digitalRead(LED_Y));
          SERIAL_DATA.print(" ");
          SERIAL_DATA.println(digitalRead(LED_G));
          break;
        default:
          SERIAL_DATA.print("ERROR: ");
          SERIAL_DATA.println(result);
      }
    }
  }
}

byte getParams(String params, byte expectedParamsCount, int * paramsOutput) {
  params.trim();
  if(params.equals("?")) return 1;
  else if(params[0] == '=') {
    for(int x = 0; x < expectedParamsCount; x++) {
      paramsOutput[x] = 0;
    }
    params = params.substring(1);
    byte temp, param = 0;
    for(byte x = 0; x < params.length(); x++) {
      if(params[x] == ' ') {
        params = params.substring(x);
        param++;
        x = 0;
        continue;
      }
      temp = params[x] - 48;
      if(temp >= 0 && temp <= 9) {
        if(param > expectedParamsCount) {
          return 2;
        }
        if(param == 0) param = 1;
        paramsOutput[param-1] = paramsOutput[param-1] * 10 + temp;
      } else {
        return 3;
      }
    }
    
    if(param == expectedParamsCount) {
      return 0;
    } else if(param < expectedParamsCount) {
      return 4;
    }
  }
  return 5;
}

